#ifndef zetsubou_ExecuteFit
#define zetsubou_ExecuteFit

//single gaussian 
TF1 *zetsubou::gfit(Double_t xmin,Double_t xmax)
{
  if(!hist) return 0; 
  func = new TF1("func",singlegaussian,xmin,xmax,3);//fitfunctionname,expression,area,number of parameters
  func->SetLineColor(fitlinecolor);
  func->SetParNames("const","mean","sigma");
  Double_t ms,mmax;
  ms= hist->GetMaximumBin()*hist->GetBinWidth(2);//position of maximumbin in the canvas
  mmax=ms + canvas->GetUxmin(); //absolute position 
  func->SetParameters(100,mmax,10);//set initial parameter
  func->SetParLimits(2,0,xmax-xmin+10);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" << func->GetNDF() << std::endl;
  std::cout << "event=" << func->GetParameter("const")  / hist->GetBinWidth(2) << std::endl;
  std::cout << "event error=" << func->GetParError(0) / hist->GetBinWidth(2) <<std::endl;
  std::cout << "mean=" << func->GetParameter("mean") << std::endl;
  std::cout << "mean error=" << func->GetParError(1) << std::endl;
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=2;i++){MakeResultBox(i,3);}
  tpt->Draw();
  return func;
}

//sigle gaussian 
TF1 *zetsubou::gfit(Double_t xmin,Double_t xmax,Double_t height,Double_t mean,Double_t sigma)
{
  if(!hist) return 0; 
  func = new TF1("func",singlegaussian,xmin,xmax,3);
  func->SetLineColor(fitlinecolor);
  func->SetParameters(height,mean,sigma);
  func->SetParNames("const","mean","sigma");
  func->SetParLimits(1,xmin,xmax);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" << func->GetNDF() << std::endl;
  std::cout << "event=" << func->GetParameter("const") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event error=" << func->GetParError(0) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean=" << func->GetParameter("mean") << std::endl;
  std::cout << "mean error=" << func->GetParError(1) << std::endl;  
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=2;i++){MakeResultBox(i,3);}
  tpt->Draw();
  return func;
}

//single gaussian + linear
TF1 *zetsubou::gfit(Double_t xmin,Double_t xmax,Double_t height,Double_t mean,Double_t sigma,Double_t katamuki ,Double_t seppen)
{
  if(!hist) return 0;   
  func = new TF1("func",singlegaussian_linear,xmin,xmax,5);
  func->SetLineColor(fitlinecolor);
  func->SetParameters(height,mean,sigma,katamuki,seppen);
  func->SetParNames("const","mean","sigma","slope","intercept");
  func->SetParLimits(1,xmin,xmax);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" << func->GetNDF() << std::endl;
  std::cout << "event=" << func->GetParameter("const") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event error=" << func->GetParError(0) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean=" << func->GetParameter("mean") << std::endl;
  std::cout << "mean error=" << func->GetParError(1) << std::endl;  
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=4;i++){MakeResultBox(i,5);}
  tpt->Draw();
  return func;
}

//double gaussian
TF1 *zetsubou::gfit(Double_t xmin,Double_t xmax,Double_t height1,Double_t mean1,Double_t sigma1,Double_t height2, Double_t mean2, Double_t sigma2)
{
  if(!hist) return 0;
  TF1 *func = new TF1("func",doublegaussian,xmin,xmax,6);
  func->SetLineColor(fitlinecolor);
  func->SetParameters(height1,mean1,sigma1,height2,mean2,sigma2);
  func->SetParNames("const1","mean1","sigma1","const2","mean2","sigma2");
  func->SetParLimits(1,xmin,xmax);
  func->SetParLimits(4,xmin,xmax);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" << func->GetNDF() << std::endl;
  std::cout << "event1=" << func->GetParameter("const1") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event1 error=" << func->GetParError(0) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean1=" << func->GetParameter("mean1") << std::endl;
  std::cout << "mean1 error=" << func->GetParError(1) << std::endl;
  std::cout << "event2=" << func->GetParameter("const2") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event2 error=" << func->GetParError(3) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean2=" << func->GetParameter("mean2") << std::endl;
  std::cout << "mean2 error=" << func->GetParError(4) << std::endl;
  
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=5;i++)
  {//{MakeResultBox(i,6);}
	if(!tpt){ tpt = new TPaveText(0.698,0.55 - 0.039*6,0.99,0.575,"NDC");
	tpt->SetFillColor(0);
	tpt->SetTextSize(0.03);
	tpt->SetTextAlign(12);}
  
	std::string sbuf1 = func->GetParName(i);
	Char_t buf[100000];
	std::sprintf(buf," : %.3f#pm%.3f",func->GetParameter(i),func->GetParError(i));
	std::string sbuf = sbuf1 + buf;
  
	tpt->AddText(sbuf.c_str());}  
  tpt->Draw();
  return func;
}

//double gaussian + linear
TF1 *zetsubou::gfit(Double_t xmin,Double_t xmax,Double_t height1,Double_t mean1,Double_t sigma1,Double_t height2, Double_t mean2, Double_t sigma2,Double_t katamuki ,Double_t seppen)
{
  if(!hist) return 0;  
  func = new TF1("func",doublegaussian_linear,xmin,xmax,8);
  func->SetLineColor(fitlinecolor);
  func->SetParameters(height1,mean1,sigma1,height2,mean2,sigma2,katamuki,seppen);
  func->SetParNames("const1","mean1","sigma1","const2","mean2","sigma2","slope","intercept");
  func->SetParLimits(1,xmin,xmax);
  func->SetParLimits(4,xmin,xmax);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" << func->GetNDF() << std::endl;
  std::cout << "event1=" << func->GetParameter("const1") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event1 error=" << func->GetParError(0) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean1="<< func->GetParameter("mean1") << std::endl;
  std::cout << "mean1 error=" << func->GetParError(1) << std::endl;
  std::cout << "event2=" << func->GetParameter("const2") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event2 error=" << func->GetParError(3) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean2=" << func->GetParameter("mean2") << std::endl;
  std::cout << "mean2 error=" << func->GetParError(4)<< std::endl;
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=7;i++){MakeResultBox(i,8);}
  tpt->Draw();
  return func;
}

//triple gaussian
TF1 *zetsubou::gfit(Double_t xmin,Double_t xmax,Double_t height1,Double_t mean1,Double_t sigma1,Double_t height2, Double_t mean2, Double_t sigma2,Double_t height3,Double_t mean3,Double_t sigma3)
{
  if(!hist) return 0;  
  func = new TF1("func",triplegaussian,xmin,xmax,9);
  func->SetLineColor(fitlinecolor);
  func->SetParameters(height1,mean1,sigma1,height2,mean2,sigma2,height3,mean3,sigma3);
  func->SetParNames("const1","mean1","sigma1","const2","mean2","sigma2","const3","mean3","sigma3");
  func->SetParLimits(1,xmin,xmax);
  func->SetParLimits(4,xmin,xmax);
  func->SetParLimits(7,xmin,xmax);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" << func->GetNDF() << std::endl;
  std::cout << "event1=" << func->GetParameter("const1") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event1 error=" << func->GetParError(0) / hist->GetBinWidth(2)<< std::endl;
  std::cout << "mean1=" << func->GetParameter("mean1") << std::endl;
  std::cout << "mean1 error=" << func->GetParError(1) << std::endl;
  std::cout << "event2=" << func->GetParameter("const2") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event2 error=" << func->GetParError(3) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean2=" << func->GetParameter("mean2") << std::endl;
  std::cout << "mean2 error=" << func->GetParError(4) << std::endl;
  std::cout << "event3=" << func->GetParameter("const3") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event3 error=" << func->GetParError(6) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean3=" << func->GetParameter("mean3") << std::endl;
  std::cout << "mean3 error=" << func->GetParError(7) << std::endl;
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=8;i++){MakeResultBox(i,9);}
  tpt->Draw();
  return func;
}

//triple gaussian + linear
TF1 *zetsubou::gfit(Double_t xmin,Double_t xmax,Double_t height1,Double_t mean1,Double_t sigma1,Double_t height2, Double_t mean2, Double_t sigma2,Double_t height3,Double_t mean3,Double_t sigma3,Double_t katamuki ,Double_t seppen)
{
  if(!hist) return 0;
  func = new TF1("func",triplegaussian_linear,xmin,xmax,11);
  func->SetLineColor(fitlinecolor);
  func->SetParameters(height1,mean1,sigma1,height2,mean2,sigma2,height3,mean3,sigma3,katamuki,seppen);
  func->SetParNames("const1","mean1","sigma1","const2","mean2","sigma2","const3","mean3","sigma3","slope","intercept");
  func->SetParLimits(1,xmin,xmax);
  func->SetParLimits(4,xmin,xmax);
  func->SetParLimits(7,xmin,xmax);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" << func->GetNDF() << std::endl;
  std::cout << "event1=" << func->GetParameter("const1") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event1 error=" << func->GetParError(0) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean1=" << func->GetParameter("mean1") << std::endl;
  std::cout << "mean1 error=" << func->GetParError(1)<< std::endl;
  std::cout << "event2=" << func->GetParameter("const2") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event2 error=" << func->GetParError(3) / hist->GetBinWidth(2)<< std::endl;
  std::cout << "mean2=" << func->GetParameter("mean2") << std::endl;
  std::cout << "mean2 error=" << func->GetParError(4) << std::endl;
  std::cout << "event3=" << func->GetParameter("const3") / hist->GetBinWidth(2) << std::endl;
  std::cout << "event3 error=" << func->GetParError(6) / hist->GetBinWidth(2) << std::endl;
  std::cout << "mean3=" << func->GetParameter("mean3") << std::endl;
  std::cout << "mean3 error=" << func->GetParError(7) << std::endl;
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=10;i++){MakeResultBox(i,11);}
  tpt->Draw();
  return func;
}

//Convoluted Landau and Gaussian Fitting 
TF1 *zetsubou::lgfit(Double_t xmin,Double_t xmax,Double_t width,Double_t mp,Double_t area,Double_t gsigma)
{
  if(!hist) return 0;
  std::cout << "Now fitting......." << std::endl;
  func = new TF1("func",langaufun,xmin,xmax,4);
  func->SetParNames("Width","MP","Area","GSigma");
  func->SetParameters(width,mp,area,gsigma);
  func->SetLineColor(fitlinecolor);
  hist->Fit("func","","",xmin,xmax);
  hist->Draw();
  std::cout << "Chisquare/NDF=" << func->GetChisquare() / func->GetNDF() << "=" << func->GetChisquare() << "/" <<func->GetNDF() << std::endl;
  if(tpt) {delete tpt;tpt=0;}
  for(Int_t i=0;i<=2;i++){MakeResultBox(i,3);}
  tpt->Draw();
  return func;
}

#endif  //ExecuteFit.h
